@@ -1,5 +1,5 @@
 /*=============================================================================
-    Copyright (c) 2001-2006 Joel de Guzman
+    Copyright (c) 2001-2011 Joel de Guzman
 
     Distributed under the Boost Software License, Version 1.0. (See accompanying 
     file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -22,7 +22,7 @@ namespace boost { namespace fusion
 
     template <typename Sequence>
     inline typename result_of::clear<Sequence const>::type
-    clear(Sequence const& seq)
+    clear(Sequence const& /*seq*/)
     {
         return vector0<>();
     }
