@@ -16,6 +16,7 @@
 #include <queue>
 #include <map>
 #include <boost/shared_ptr.hpp>
+#include <boost/assert.hpp>
 #include <boost/random/uniform_int.hpp>
 #include <boost/random/uniform_01.hpp>
 #include <boost/graph/graph_traits.hpp>
@@ -155,7 +156,7 @@ namespace boost {
     {
       this->gen.reset(new uniform_01<RandomGenerator>(gen));
 
-      assert(boost::test_tools::check_is_close(a + b + c + d, 1., boost::test_tools::fraction_tolerance(1.e-5)));
+      BOOST_ASSERT(boost::test_tools::check_is_close(a + b + c + d, 1., boost::test_tools::fraction_tolerance(1.e-5)));
 
       if (permute_vertices)
         generate_permutation_vector(gen, vertexPermutation, n);
@@ -164,7 +165,7 @@ namespace boost {
 
       // Generate the first edge
       vertices_size_type u, v;
-      tie(u, v) = generate_edge(this->gen, n, SCALE, a, b, c, d);
+      boost::tie(u, v) = generate_edge(this->gen, n, SCALE, a, b, c, d);
 
       if (permute_vertices)
         current = std::make_pair(vertexPermutation[u],
@@ -181,7 +182,7 @@ namespace boost {
     rmat_iterator& operator++()
     {
       vertices_size_type u, v;
-      tie(u, v) = generate_edge(this->gen, n, SCALE, a, b, c, d);
+      boost::tie(u, v) = generate_edge(this->gen, n, SCALE, a, b, c, d);
 
       if (permute_vertices)
         current = std::make_pair(vertexPermutation[u],
@@ -265,7 +266,7 @@ namespace boost {
         values(sort_pair<vertices_size_type>()), done(false)
 
     {
-      assert(boost::test_tools::check_is_close(a + b + c + d, 1., boost::test_tools::fraction_tolerance(1.e-5)));
+      BOOST_ASSERT(boost::test_tools::check_is_close(a + b + c + d, 1., boost::test_tools::fraction_tolerance(1.e-5)));
 
       this->gen.reset(new uniform_01<RandomGenerator>(gen));
 
@@ -279,7 +280,7 @@ namespace boost {
       for (edges_size_type i = 0; i < m; ++i) {
 
         vertices_size_type u, v;
-        tie(u, v) = generate_edge(this->gen, n, SCALE, a, b, c, d);
+        boost::tie(u, v) = generate_edge(this->gen, n, SCALE, a, b, c, d);
 
         if (permute_vertices) {
           if (ep(vertexPermutation[u], vertexPermutation[v]))
@@ -366,7 +367,7 @@ namespace boost {
       : gen(), done(false)
 
     {
-      assert(boost::test_tools::check_is_close(a + b + c + d, 1., boost::test_tools::fraction_tolerance(1.e-5)));
+      BOOST_ASSERT(boost::test_tools::check_is_close(a + b + c + d, 1., boost::test_tools::fraction_tolerance(1.e-5)));
 
       this->gen.reset(new uniform_01<RandomGenerator>(gen));
 
@@ -381,7 +382,7 @@ namespace boost {
       edges_size_type edges = 0;
       do {
         vertices_size_type u, v;
-        tie(u, v) = generate_edge(this->gen, n, SCALE, a, b, c, d);
+        boost::tie(u, v) = generate_edge(this->gen, n, SCALE, a, b, c, d);
 
         // Lowest vertex number always comes first
         // (this means we don't have to worry about i->j and j->i being in the edge list)
@@ -479,7 +480,7 @@ namespace boost {
         values(sort_pair<vertices_size_type>()), done(false)
 
     {
-      assert(boost::test_tools::check_is_close(a + b + c + d, 1., boost::test_tools::fraction_tolerance(1.e-5)));
+      BOOST_ASSERT(boost::test_tools::check_is_close(a + b + c + d, 1., boost::test_tools::fraction_tolerance(1.e-5)));
 
       this->gen.reset(new uniform_01<RandomGenerator>(gen));
 
@@ -495,7 +496,7 @@ namespace boost {
       do {
 
         vertices_size_type u, v;
-        tie(u, v) = generate_edge(this->gen, n, SCALE, a, b, c, d);
+        boost::tie(u, v) = generate_edge(this->gen, n, SCALE, a, b, c, d);
 
         if (bidirectional) {
           if (edge_map.find(std::make_pair(u, v)) == edge_map.end()) {
